#include <iostream>
#include <type_traits>

struct tracer {
	std::ostream & out;
	tracer( std::ostream & out, char const * file, int line )
		: out( out ) {
		out << file << ":" << line << ": ";
	}
	~tracer() {
		out << std::endl;
	}
	
	template<typename TF, typename ... TR>
	void write( TF&& f, TR&& ... rest ) {
		write_debug_output( out, std::forward<TF>(f) );
		out << " ";
		write( std::forward<TR>(rest)... );
	}
	template<typename TF>
	void write( TF&& f ) {
		write_debug_output( out, std::forward<TF>(f) );
	}
	void write() {
		//handle the empty params case
	}
};

template<typename TF>
void write_debug_output( std::ostream & out, TF&& f ) {
	out << f;
}

#define TRACE(...) tracer( std::cout, __FILE__, __LINE__ ).write( __VA_ARGS__ )

struct my_object { };
void write_debug_output( std::ostream & out,  my_object const & f ) {
	out << "my_object const &";
}
void write_debug_output( std::ostream & out,  my_object & f ) {
	out << "my_object &";
}
void write_debug_output( std::ostream & out,  my_object && f ) {
	out << "my_object &&";
}

int main()
{
 	my_object const a;
 	TRACE("CONST",a);
 	my_object b;
 	TRACE("MUTABLE",b);
	TRACE("RVALUE", my_object() );
	TRACE( 123, 5.5 );
	TRACE();
}
