#ifndef DSP_HPP
#define DSP_HPP

#include <cstdlib>
#include <vector>

typedef std::vector<float> audio_vector_t;

namespace dsp
{

template<typename T>
inline T sqr( T val )
{
	return val * val;
}

inline float power_filter( float const * buf, float const * fi, size_t len )
{
	float s = 0;

	for( size_t i=0; i < len; ++i )
		s += (*buf++) * (*fi++);
	return s;
}

inline float fourier_power_coefficient( float const * buf, float const * i, float const * q, size_t len )
{
	return sqr( power_filter( buf, i, len ) )
		+ sqr( power_filter( buf, q, len ) );
}

/**
	Generate a constant tone for the given sample length.
*/
void gen_tone( audio_vector_t & av, size_t len, double freq, double sample_freq, double phase  = 0 );

/**
	A simple I/Q filter to detect frequency power in audio data.
*/
struct iq_filter
{
	audio_vector_t av_i, av_q;

	iq_filter()
	{ }
	iq_filter( double freq, double sample_freq, size_t len )
	{ construct( freq, sample_freq, len ); }

	void construct( double freq, double sample_freq, size_t len );

	double fourier_power_coefficient( float const * buf ) const
	{
		return dsp::fourier_power_coefficient( buf, &av_i[0], &av_q[0], av_i.size() );
	}
};

} //eon dsp

#endif