#ifndef MEM_STREAM_HPP
#define MEM_STREAM_HPP

#include "modem_stream.hpp"

/**
	A highly inefficient, but useful for testing, output stream.
*/
struct memory_ostream : public modem::output_stream
{
	audio_vector_t data;

	void write( float * buf, size_t len )
	{
		data.insert( data.end(), buf, buf + len );
	}

};

/**
	A simple stream for reading from in-memory data. Used for testing.
*/
struct memory_istream : public modem::input_stream
{
	audio_vector_t data;
	size_t at;

	memory_istream()
		: at( 0 )
	{
	}

	bool read( float * buf, size_t len, size_t offset )
	{
		size_t begin = at + offset;
		size_t end = begin + len;
		size_t trunc_len = begin >= data.size() ? 0 : (std::min( end, data.size() ) - begin);

		std::copy( &data[begin], &data[begin] + trunc_len, buf );

		if( trunc_len == len )
			return true;

		//pad with 0
		std::fill( buf + trunc_len, buf + len, 0 );
		return false;
	}

	void advance( size_t len )
	{
		at += len;
	}

	bool exhausted()
	{
		return at >= data.size();
	}
};


#endif