#include "modem.hpp"

#include <iostream>

void power_levels();

int main()
{
	power_levels();
}


typedef std::vector<float> audio_vector_t;

double scan_power_level( dsp::iq_filter const & iq, audio_vector_t const & av )
{
	double sum = 0;
	for( size_t i=0; i < iq.av_i.size(); ++i )
		sum += iq.fourier_power_coefficient( &av[i] );

	return sum / double(iq.av_i.size());
}

/**
	Check the natural power level difference between the frequencies
	to get the default gain.
*/
void power_levels()
{
	double sample_freq = 22050;

	//the signal length should be the length of a symbol (baud rate)
	size_t cor_len = size_t(sample_freq / 1200);
	
	audio_vector_t av_1200, av_2200, av_12_22, av_22_12;

	dsp::gen_tone( av_1200, cor_len * 2, 1200, sample_freq );
	dsp::gen_tone( av_2200, cor_len * 2, 2200, sample_freq );
	//transition tones, typical for AFSK
	dsp::gen_tone( av_12_22, cor_len / 2, 1200, sample_freq );
	dsp::gen_tone( av_12_22, cor_len, 2200, sample_freq );
	dsp::gen_tone( av_12_22, cor_len / 2, 1200, sample_freq );

	dsp::gen_tone( av_22_12, cor_len / 2, 2200, sample_freq );
	dsp::gen_tone( av_22_12, cor_len, 1200, sample_freq );
	dsp::gen_tone( av_22_12, cor_len / 2, 2200, sample_freq );
	
	dsp::iq_filter filt_1200( 1200, sample_freq, cor_len ),
		filt_2200( 2200, sample_freq, cor_len );

	#define POWER_LEVEL( iq_, av_ ) \
		double iq_##_##av_##_power = scan_power_level( iq_, av_ ); \
		std::cout << #iq_ "/" #av_ ": " << iq_##_##av_##_power << std::endl;

	POWER_LEVEL( filt_1200, av_1200 )
	POWER_LEVEL( filt_1200, av_2200 )
	POWER_LEVEL( filt_1200, av_12_22 )
	POWER_LEVEL( filt_1200, av_22_12 )
	std::cout << std::endl;
	POWER_LEVEL( filt_2200, av_1200 )
	POWER_LEVEL( filt_2200, av_2200 )
	POWER_LEVEL( filt_2200, av_12_22 )
	POWER_LEVEL( filt_2200, av_22_12 )
	std::cout << std::endl;

	double gain_1200 = filt_1200_av_22_12_power / filt_2200_av_12_22_power;
	std::cout << "Required Gain 1200: " << gain_1200 << std::endl;
	
	#undef POWER_LEVEL
}
