#include "modem_codec.hpp"

extern "C" {
#include "rs.h"
}

#include <cstring>

using namespace std;

namespace modem
{

/**
	Wrap the Reed-Solomon codec from the ZBar package.
*/
struct rs_codec : public block_codec
{
	size_t block_len;

	size_t parity;
	size_t data_len;

	rs_gf256 gf;
    unsigned char genpoly[256];

	rs_codec( size_t parity, size_t block )
		: block_len( block )
		, parity( parity )
	{
		//TODO: check parity size/block size
		data_len = block_len - parity;

		rs_gf256_init( &gf, QR_PPOLY );
	    rs_compute_genpoly(&gf, QR_M0, genpoly, int(parity) );
	}

	size_t get_encoded_block_len() const
	{ return block_len; }
	size_t get_data_block_len() const
	{ return data_len; }

	uint8_t decode( uint8_t const * enc_data, size_t const * erasure, size_t erasure_sz,
		uint8_t * dec_data )
	{
		uint8_t buf[block_len];
		memcpy( buf, enc_data, block_len );

		unsigned char cer[erasure_sz];
		for( size_t i=0; i < erasure_sz; ++i )
			cer[i] = (unsigned char)(erasure[i]);

		int res = rs_correct( &gf, QR_M0, buf, int(block_len), int(parity), cer, int(erasure_sz) );
		if( res < 0 )
			return 255; //completely wrong

		memcpy( dec_data, buf, data_len );
		return 0;
	}

	void encode( uint8_t const * dec_data, uint8_t * enc_data )
	{
		memcpy( enc_data, dec_data, data_len );
		rs_encode( &gf, enc_data, int(block_len), genpoly, int(parity) );
	}
};

auto_ptr<block_codec> create_rs_codec( size_t parity, size_t block )
{
	return auto_ptr<block_codec>( new rs_codec( parity, block ) );
}

}
