#ifndef MODEM_CODEC_HPP
#define MODEM_CODEC_HPP

#include <stdint.h>
#include <memory>

namespace modem
{

struct block_codec
{
	virtual size_t get_encoded_block_len() const = 0;
	virtual size_t get_data_block_len() const = 0;

	/**
		Decodes a block of data.

		The return value is used to help locate another decoding logic whether the
		block is a proper block, but just too corrupted to be recovered, or whether
		it is just a wrong block.

		@param enc_data the raw, expected to have get_encoded_block_len data available
		@param dec_data where to write the output, expected to be get_data_block_len in size

		@return
			0 = successfully decoded the data block
			x/255 chance this is *not* a valid data block (so 255 is a guarantee this is not
				a valid data block)
	*/
	virtual uint8_t decode( uint8_t const * enc_data, size_t const * erasure, size_t erasure_sz,
		uint8_t * dec_data ) = 0;

	/**
		Encodes a block of data.
	*/
	virtual void encode( uint8_t const * dec_data, uint8_t * enc_data ) = 0;
};

std::auto_ptr<block_codec> create_rs_codec( size_t parity, size_t block = 255 );

} //eon modem

#endif