#ifndef MODEM_STREAM_HPP
#define MODEM_STREAM_HPP

#include <stdint.h>

namespace modem
{

/**
	Interface used to write output audio data.
*/
struct output_stream
{
	virtual void write( float * buf, size_t len ) = 0;
};

/**
	Interface to read audio data.
*/
struct input_stream
{
	/**
		Read the desired number of bytes to buf. If there is not enough data
		left than pad with 0 and return false, otherwise return true.

		Reading happens from the current cursor location + offset. The buffer
		must be large enough to allow navigation within the next several
		bytes worth of decoded data.
	*/
	virtual bool read( float * buf, size_t len, size_t offset = 0 ) = 0;

	/**
		Advances the current cursor location.
	*/
	virtual void advance( size_t len ) = 0;

	/**
		@return true if there is no data left (reading would reutrn only 0 pad)
	*/
	virtual bool exhausted()  = 0;
};

} //eon modem

#endif