#include "modem_codec.hpp"

#include <iostream>
#include <cstring>
#include <cstdlib>
#include <set>

using namespace std;

int main()
{
	const int buf_len = 256;
	uint8_t buffer[buf_len+1], enc_buffer[buf_len+1]; //include extra zero to terminate any length of block
	memset( buffer, 0, sizeof(buffer) );
	memset( enc_buffer, 0, sizeof(enc_buffer) );
	strcpy( (char*)buffer, "This is a longer sample message to test Reed Solomon FEC. 01234567890." );

	int seed = int(time(0));
	cout << "Seed: " << seed << endl;
	srand( seed );
	
	cout << "I: " << buffer << endl;

	int parity = 64;
	auto_ptr<modem::block_codec> rsd( modem::create_rs_codec( parity ) );
	rsd->encode( buffer, enc_buffer );
    
	//RS requires 2-parity to correct 1-corruption, a 1-parity to correct 1 erasure
	
	//mess up a few locations
	size_t corrupt_sz = parity / 4;
	for( size_t i=0; i < corrupt_sz; ++i )
	{
		size_t loc = rand() % rsd->get_encoded_block_len();
		enc_buffer[ loc ] = (unsigned char)(rand());
	}

	//record some erasures
	size_t erasure_sz = parity / 2;
	size_t erasure[erasure_sz];
	set<size_t> in_erasure; //the algorithm really doesn't appreciate duplicate erasures points
	for( size_t i=0; i < erasure_sz; ++i )
	{
		size_t loc;
		do
		{
			loc = rand() % rsd->get_encoded_block_len();
		} while( in_erasure.find( loc ) != in_erasure.end() );
		in_erasure.insert( loc );
			
		enc_buffer[loc] = '*';
		erasure[i] = loc;
	}
	
	cout << "C: " << enc_buffer << endl;

	uint8_t ret = rsd->decode( enc_buffer, erasure, erasure_sz, buffer );
	cout << "O: " << (ret != 0 ? "Error" : (char*)buffer ) << endl;
}
