#include "perf/timer.hpp"

#include <boost/program_options.hpp>
namespace po = boost::program_options;

using namespace std;

template<typename T>
void run_multiply( size_t size, size_t start, size_t loop_size )
{
	perf::timer t;
	
	//run loop
	T end = T(T(start) + T(size));
	if( end < start )
	{
		cerr << "Size/start too large, wraps around" << endl;
		return;
	}

	t.start();
	T total = 0;
	for( size_t l=0; l < loop_size; ++l )
	{
		for( T i=T(start); i < end; ++i )
		{
			for( T j=T(start); j < end; ++j )
			{
				total += T(i * j);
			}
		}
		t.end();
	}
	t.consume( total );
	
	t.report( typeid(T).name() );
}

int main( int argc, char ** argv )
{
	//collect options
	size_t size_log;
	size_t start_log;
	size_t loop_size;
	
	po::options_description desc( "Options" );
	desc.add_options()
		( "help", "show help" )
		( "size", po::value<size_t>( &size_log )->default_value(8), "Range size to multiply (2^N)" )
		( "start", po::value<size_t>( &start_log )->default_value(0), "Starting value (2^N)" )
		( "loop", po::value<size_t>( &loop_size )->default_value(1000), "Number of iterations" );
		;
	
	po::variables_map vm;
	po::store( po::command_line_parser( argc, argv ).options( desc ).run(), vm );
	po::notify( vm );
	
	if( vm.count( "help" ) )
	{
		cerr << desc;
		return 1;
	}
	
	//translate sizes
	size_t size = size_t(2) << size_log;
	size_t start = size_t(2) << start_log;
	
	cout << "Multiply from " << start << " to +" << size << endl;
	run_multiply<uint16_t>( size, start, loop_size );
	run_multiply<uint32_t>( size, start, loop_size );
	run_multiply<uint64_t>( size, start, loop_size );
}
