#include "perf/quick_stats.hpp"
#include "perf/timer.hpp"

#include <boost/test/unit_test.hpp>

BOOST_AUTO_TEST_SUITE( perf_suite )

BOOST_AUTO_TEST_CASE( quick_stats )
{
	perf::quick_stats qs;
	
	for( int i=0; i <= 10; ++i )
		qs.add( i );
		
	BOOST_CHECK_EQUAL( 11, qs.count() );
	BOOST_CHECK_CLOSE( 5.0, qs.mean(), 1e-9 );
	BOOST_CHECK_CLOSE( 0, qs.min(), 1e-9 );
	BOOST_CHECK_CLOSE( 10, qs.max(), 1e-9 );
	BOOST_CHECK_CLOSE( 11.0, qs.variance(), 1e-9 );
	//hmm, though it was an approx, seems pretty exact to me...
	BOOST_CHECK_CLOSE( 3.3166247903553998100e+00, qs.std_dev(), 1e-9 );
}

BOOST_AUTO_TEST_CASE( timer )
{
	perf::timer t;

	for( int i=0; i < 1000; ++i )
	{
		//you have to do soemthing otherwise the loop might be optimized away
		t.consume(i);
		t.end();
	}
		
	t.report( "Nothing" );
	//we can't really check timings, just ensuring it compiles/output looks okay
	//the above report is thus the overhead of the timer
	BOOST_CHECK( true );
}

BOOST_AUTO_TEST_SUITE_END()
